(* contexts always assume entry names are full paths *)

signature CONTEXT =
sig
  structure Throttle : THROTTLE
  structure Entry : ENTRY
  structure AcapTime : ACAPTIME

  exception Modified of AcapTime.acaptime

  datatype outMsg = 
    CNEW of string * Entry.entry * int option
  | CCHANGE of string * Entry.entry * (int * int) option
  | CDELETE of string * Entry.entry * int option
  | CTIME of AcapTime.acaptime
  | CDIE

  type context

  val create : Entry.Acl.Auth.ident -> (Entry.entry -> bool)
    -> ((string * Entry.entry) * (string * Entry.entry) 
	-> bool) option * outMsg CML.chan option * bool -> context

  (* no notifications; doesn't resort until "doneInitial" is called *)
  val fastInsert : context -> string * Entry.entry -> unit

  (* has side-effect of starting a thread to listen for changes *)
  val doneInitial : context -> unit

  (* attempt to insert new entry into context; give notification *)
  val insert : context -> string * Entry.entry -> unit

  (* change an entry that may or may not be in the context *)
  val change : context -> (string * Entry.entry) * (string * Entry.entry) 
    -> unit

  (* delete an entry that may or may not be in the context *)
  val delete : context -> (string * Entry.entry) -> unit

  (* ask the context to send a CTIME message *)
  val updatecontext : context -> unit
    
  (* search the current context *)
  val search : context -> ((int option * Entry.entry) -> bool) * 
    (string * Entry.entry) Throttle.throttle * AcapTime.acaptime -> unit

  (* add a new dataset to listen to *)
  val addDSet : context -> Entry.updatemsg RMulticast.port -> unit

  (* done with the context *)
  val destroy : context -> unit
end
